
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include "system.h"
#include "sample.h"
#include "sys/alt_timestamp.h"
#include "alt_types.h"
#include "sys/alt_irq.h"
#include "btc_dprx_syslib.h"
#include "btc_dptx_syslib.h"
#include "debug.h"
#include "i2c.h"
#include "config.h"

//========//
// Grobal //
//========//
alt_u32			AVB_FRM_WT_BASE[1] = { CH1_AVB_FRM_WT_BASE } ;
alt_u32			AVB_FRM_RD_BASE[1] = { CH1_VIP_VFR_BASE } ;
alt_u32			CH_OFFSET_ADR[1] = { CH1_OFFSET_ADR } ;
bank_status_t	G_Bank_Status[1] = { { 0 ,0 ,0 ,{ 0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 } } } ;

alt_u8			G_Base_Bank[1] = { 0 } ;
alt_u8			ptp_state = 0 ;


//==============================//
// Display Port initial Setting //
//==============================//
void display_port_init( ) {
	fcntl(STDOUT_FILENO, F_SETFL, O_NONBLOCK);
	printf("DP Started...\n");

	// Init Bitec DP system library
	btc_dptx_syslib_init(DP_BASE,DP_IRQ_INTERRUPT_CONTROLLER_ID,DP_IRQ) ;
	
	// Init sink and source
	bitec_dptx_init();

//	// bitec_dp_dump_aux_debug_init(AUX_RX_DEBUG_FIFO_IN_CSR_BASE);
//	bitec_dp_dump_aux_debug_init(AUX_TX_DEBUG_FIFO_IN_CSR_BASE);

	// Perform Link Training if a Sink is connected
	{
		unsigned int sr;
		sr = IORD(btc_dptx_baseaddr(),DPTX_REG_TX_STATUS); // Reading SR clears IRQ
		if(sr & 0x04)
			bitec_dptx_linktrain();
	}

	BTC_DPTX_ENABLE_HPD_IRQ(); // Enable IRQ on HPD changes from the sink

}
	
//=========================//
// MAC PHY initial Setting //
//=========================//
void mac_init(  alt_u32 mac_base_adr , alt_u8 board_no ) {
	alt_u32	mac_adr_top ;
	alt_u32	mac_adr_low ;

	if ( board_no == 0x00 ) {
		mac_adr_top = MACSET_CH1_ADR_TOP ;
		mac_adr_low = MACSET_CH1_ADR_LOW ;
	}
	else {
		mac_adr_top = MACSET_CH2_ADR_TOP ;
		mac_adr_low = MACSET_CH2_ADR_LOW ;
	}
		
	// MAC
//	IOWR( mac_base_adr , 0x0A , 0x00000003 ) ;
	IOWR( mac_base_adr , 0x02 , 0x00000203 ) ;
	IOWR( mac_base_adr , 0x03 , mac_adr_top ) ;
	IOWR( mac_base_adr , 0x04 , mac_adr_low ) ;
	IOWR( mac_base_adr , 0x07 , 0x000003F0 ) ;
	IOWR( mac_base_adr , 0x09 , 0x000003F0 ) ;
//	IOWR( mac_base_adr , 0x0A , 0x00000003 ) ;
//	IOWR( mac_base_adr , 0x0B , 0x00000008 ) ;
//	IOWR( mac_base_adr , 0x0C , 0x00000008 ) ;
//	IOWR( mac_base_adr , 0x0D , 0x00000008 ) ;
//	IOWR( mac_base_adr , 0x0E , 0x00000003 ) ;
//	IOWR( mac_base_adr , 0x0F , 0x00000002 ) ;

}

//================//
// PHY Initialize //
//================//
//== BCM898110 ==
void bcm898110_init ( alt_u32 mac_base_adr , alt_u8 board_no ) {

	IOWR( mac_base_adr , 0x10 ,  0x00000000		) ;

	IOWR( mac_base_adr , 0xA0+MIICTL ,  0x8000	) ;	

	IOWR( mac_base_adr , 0xA0+DSPADDR, EXPREG93	) ;	
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x107F	) ;
	IOWR( mac_base_adr , 0xA0+DSPADDR , EXPREG90	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x0001	) ;
	IOWR( mac_base_adr , 0xA0+MIICTL ,  0x3000	) ;
	IOWR( mac_base_adr , 0xA0+MIICTL ,  0x0200	) ;

	IOWR( mac_base_adr , 0xA0+AUXCTRL , 0x0C00	) ;

	IOWR( mac_base_adr , 0xA0+DSPADDR , EXPREG90	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x0000	) ;
	IOWR( mac_base_adr , 0xA0+MIICTL ,  0x0100	) ;

	IOWR( mac_base_adr , 0xA0+DSPADDR , 0x0001	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x0027	) ;

	IOWR( mac_base_adr , 0xA0+DSPADDR , 0x000E	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x9B52	) ;

	IOWR( mac_base_adr , 0xA0+DSPADDR , 0x000F	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0xA04D	) ;

	IOWR( mac_base_adr , 0xA0+DSPADDR , EXPREG90	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x0001	) ;

	IOWR( mac_base_adr , 0xA0+DSPADDR , EXPREG92	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x9225	) ;

	IOWR( mac_base_adr , 0xA0+DSPADDR , 0x000A	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x0323	) ;

	IOWR( mac_base_adr , 0xA0+DSPADDR , EXPREGFD	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x1C3F	) ;

	IOWR( mac_base_adr , 0xA0+DSPADDR , EXPREGFE	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x1C3F	) ;

	IOWR( mac_base_adr , 0xA0+DSPADDR , EXPREG99	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x7180	) ;
	IOWR( mac_base_adr , 0xA0+DSPADDR , EXPREG9A	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x34C0	) ;


	IOWR( mac_base_adr , 0xA0+DSPADDR , EXPREG0E	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x0000	) ;	
	IOWR( mac_base_adr , 0xA0+DSPADDR , EXPREG9F	) ;
	IOWR( mac_base_adr , 0xA0+DSPRW ,   0x0000	) ;	
	IOWR( mac_base_adr , 0xA0+AUXCTRL , 0xF0E7	) ;	

	if ( board_no ==0x00 ) {
		IOWR( mac_base_adr , 0xA0+MIICTL , 0x0208	) ;
	}
	else {
		IOWR( mac_base_adr , 0xA0+MIICTL , 0x0200	) ;
	}
	
}

//== BCM898111 ==
void bcm898111_init ( alt_u32 mac_base_adr , alt_u8 board_no ) {
	alt_u32 data0 ;

	IOWR( mac_base_adr , 0x10 ,  0x00000001	) ;

	IOWR( mac_base_adr , 0xA0+MIICTL ,  0x8000	) ;	

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x0028	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0x0C00	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x0312	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0x030B	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x030A	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0x34C0	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x0166	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0x0020	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x012D	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0x9B52	) ;
	IOWR( mac_base_adr , 0xA0+0x1E ,    0x012E	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0xA04D	) ;

	data0	= 0x0000 || ( MII_PAD_SETTING * 2048 ) ;
	IOWR( mac_base_adr , 0xA0+0x1E ,    0x0811	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    data0	) ;

	data0	= 0x0000 || ( TXSWAG * 64 ) || ( RXFILTER * 8 ) || ( TXFILTER ) ;
	IOWR( mac_base_adr , 0xA0+0x1E ,    0x01D3	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    data0	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x01C1	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0xA5F7	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x0028	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0x0400	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x001D	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0x3411	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x0820	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0x3411	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x0820	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0x0401	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x0045	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0x0000	) ;

	IOWR( mac_base_adr , 0xA0+0x1E ,    0x002F	) ;
	IOWR( mac_base_adr , 0xA0+0x1F ,    0xF1E7	) ;

	if ( board_no ==0x00 ) {
		IOWR( mac_base_adr , 0xA0+MIICTL , 0x0208	) ;
	}
	else {
		IOWR( mac_base_adr , 0xA0+MIICTL , 0x0200	) ;
	}

}

//=========================//
// Nwtwork initial Setting //
//=========================//
void netwrok_init( alt_u8 board_no ) {

	// <<< MAC & PHY setting >>>
printf( "<<< MAC & PHY setting >>>\n" ) ;
	mac_init( TSE_MAC_BASE , board_no ) ;
	bcm898110_init( TSE_MAC_BASE , board_no ) ;
	bcm898111_init( TSE_MAC_BASE , board_no ) ;

}

//==============================//
// video stream initial setting //
//==============================//
void stream_init( alt_u8 board_no ) {
	alt_u32	dest_adr_top ;
	alt_u32	dest_adr_low ;
	alt_u32	src_adr_top ;
	alt_u32	src_adr_low ;

	if ( board_no == 0x00 ) {
		dest_adr_top = PKTASM_CH2_ADR_TOP ;
		dest_adr_low = PKTASM_CH2_ADR_LOW ;
		src_adr_top  = PKTASM_CH1_ADR_TOP ;
		src_adr_low  = PKTASM_CH1_ADR_LOW ;
	}
	else {
		dest_adr_top = PKTASM_CH1_ADR_TOP ;
		dest_adr_low = PKTASM_CH1_ADR_LOW ;
		src_adr_top  = PKTASM_CH2_ADR_TOP ;
		src_adr_low  = PKTASM_CH2_ADR_LOW ;
	}
		
	printf("------ AVB Packet assembler set start. ------ \n");
	// <<< TX0 AVB packet Assembler >>>
	IOWR( AVB_PCK_ASM_BASE , 0x00 , 0x00000000 ) ;			// Reset Release
	IOWR( AVB_PCK_ASM_BASE , 0x01 , PACKET_SIZE ) ;			// Packet Size = 1KByte
	IOWR( AVB_PCK_ASM_BASE , 0x02 , dest_adr_top ) ;		// Destination Address
	IOWR( AVB_PCK_ASM_BASE , 0x03 , dest_adr_low ) ;		// 
	IOWR( AVB_PCK_ASM_BASE , 0x04 , src_adr_top ) ;			// Source Address
	IOWR( AVB_PCK_ASM_BASE , 0x05 , src_adr_low ) ;			// 
	IOWR( AVB_PCK_ASM_BASE , 0x06 , 0x81000002 ) ;			// VLAN Tag
	IOWR( AVB_PCK_ASM_BASE , 0x07 , 0x0000EEEE ) ;			// Ethernet Type
	IOWR( AVB_PCK_ASM_BASE , 0x08 , 0x00000001 ) ;			// Transmit
	printf("------ set end. ------ \n");

}

//=================================//
// AVB Frame Write initial Setting //
//=================================//
void set_rx_avb_frm_wt_top ( alt_u32 base_adr ,alt_u32 offset_adr ,alt_u32 source_adr_top ,alt_u32 source_adr_low ) {
	IOWR( base_adr , AVB_FRM_SRC_ADR_H , source_adr_top ) ;				// Target Source Address
	IOWR( base_adr , AVB_FRM_SRC_ADR_L , source_adr_low ) ;				// Target Source Address
	IOWR( base_adr , AVB_FRM_PKT_SIZE  , PACKET_SIZE ) ;				// RX Packet Size = 1KByte
	IOWR( base_adr , AVB_FRM_BK1_OF_ADR , offset_adr + BANK1_ADR ) ;	// Bank1 Offset address
	IOWR( base_adr , AVB_FRM_BK2_OF_ADR , offset_adr + BANK2_ADR ) ;	// Bank2 Offset address
	IOWR( base_adr , AVB_FRM_BK3_OF_ADR , offset_adr + BANK3_ADR ) ;	// Bank3 Offset address
	IOWR( base_adr , AVB_FRM_BK4_OF_ADR , offset_adr + BANK4_ADR ) ;	// Bank4 Offset address
	IOWR( base_adr , AVB_FRM_BK5_OF_ADR , offset_adr + BANK5_ADR ) ;	// Bank5 Offset address
	IOWR( base_adr , AVB_FRM_BK6_OF_ADR , offset_adr + BANK6_ADR ) ;	// Bank6 Offset address
	IOWR( base_adr , AVB_FRM_BK7_OF_ADR , offset_adr + BANK7_ADR ) ;	// Bank7 Offset address
	IOWR( base_adr , AVB_FRM_BK8_OF_ADR , offset_adr + BANK8_ADR ) ;	// Bank8 Offset address
	IOWR( base_adr , AVB_FRM_INT_MASK , 0x00000000 ) ;					// Interrupt Mask
}

//==================================//
// AVB Frame Reader initial Setting //
//==================================//
void set_rx_vip_vfr ( alt_u32 base_adr ,alt_u32 offset_adr ) {
	IOWR( base_adr ,0x03 ,0 ) ;											// Frame Select
	IOWR( base_adr ,0x04 ,offset_adr ) ;								// Frame 0 Address
	IOWR( base_adr ,0x05 ,(CLIP_X_SIZE * CLIP_Y_SIZE) / (256 / 16) ) ;	// (XxY)/(BUS_DATA_BIT/PIXEL_BIT) = (CLIP_X_SIZEpixel X CLIP_Y_SIZEpixel) / (256bit / 16bit)
	IOWR( base_adr ,0x06 ,(CLIP_X_SIZE * CLIP_Y_SIZE)) ;				// Frame Single Cycle Color Patterns (CLIP_X_SIZEpixel X CLIP_Y_SIZEpixel)
	IOWR( base_adr ,0x08 ,CLIP_X_SIZE ) ;								// Frame Width = CLIP_X_SIZE
	IOWR( base_adr ,0x09 ,CLIP_Y_SIZE ) ;								// Frame Height = CLIP_Y_SIZE
	IOWR( base_adr ,0x0A ,0x00000000 ) ;								// Frame Interlaced = 0

	IOWR( base_adr ,0x0B ,offset_adr ) ;								// Frame 1 Address
	IOWR( base_adr ,0x0C ,(CLIP_X_SIZE * CLIP_Y_SIZE) / (256 / 16) ) ;	// (XxY)/(BUS_DATA_BIT/PIXEL_BIT) = (CLIP_X_SIZEpixel X CLIP_Y_SIZEpixel) / (256bit / 16bit)
	IOWR( base_adr ,0x0D ,(CLIP_X_SIZE * CLIP_Y_SIZE)) ;				// Frame Single Cycle Color Patterns (CLIP_X_SIZEpixel X CLIP_Y_SIZEpixel)
	IOWR( base_adr ,0x0F ,CLIP_X_SIZE ) ;								// Frame Width = CLIP_X_SIZE
	IOWR( base_adr ,0x10 ,CLIP_Y_SIZE ) ;								// Frame Height = CLIP_Y_SIZE
	IOWR( base_adr ,0x11 ,0x00000000 ) ;								// Frame Interlaced = 0
}

//====================//
// display initialize //
//====================//
void display_init ( alt_u8 board_no ) {
	alt_u32	src_mac_adr_top ;
	alt_u32	src_mac_adr_low ;

	if ( board_no == 0x00 ) {
		src_mac_adr_top = FRMWT_CH2_ADR_TOP ;
		src_mac_adr_low = FRMWT_CH2_ADR_LOW ;
	}
	else {
		src_mac_adr_top = FRMWT_CH1_ADR_TOP ;
		src_mac_adr_low = FRMWT_CH1_ADR_LOW ;
	}
		
	// <<< AVB Frame Write Setting >>>
printf( "<<< AVB Frame Write Setting >>>\n" ) ;
	set_rx_avb_frm_wt_top ( CH1_AVB_FRM_WT_BASE ,CH1_OFFSET_ADR ,src_mac_adr_top ,src_mac_adr_low ) ;

	// <<< Frame Reader Setting >>>
	set_rx_vip_vfr ( CH1_VIP_VFR_BASE ,CH1_OFFSET_ADR ) ;

}

//================================================//
// rd_bank_status
//================================================//
alt_u8 rd_bank_status ( alt_u8 ch_number ) {
	alt_u8	i ;
	alt_u8	wt_bank_status ;
	alt_u32	time_sec_tmp ;
	alt_u32	time_nsec_tmp ;

	IOWR( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_INT_STATUS ,0x00000001 ) ;	// interrupt clear
	wt_bank_status = (alt_u8)IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_WT_BANK ) ;
	
	if ( wt_bank_status == 0 ) {
		wt_bank_status = 7 ;
	}
	else {
		wt_bank_status = wt_bank_status - 1 ;
	}

	switch ( wt_bank_status ) {
		case 1 :
			time_sec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK1_SEC ) ; //
			time_nsec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK1_NS ) ; //
		break ;
		case 2 :
			time_sec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK2_SEC ) ; //
			time_nsec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK2_NS ) ; //
		break ;
		case 3 :
			time_sec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK3_SEC ) ; //
			time_nsec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK3_NS ) ; //
		break ;
		case 4 :
			time_sec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK4_SEC ) ; //
			time_nsec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK4_NS ) ; //
		break ;
		case 5 :
			time_sec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK5_SEC ) ; //
			time_nsec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK5_NS ) ; //
		break ;
		case 6 :
			time_sec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK6_SEC ) ; //
			time_nsec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK6_NS ) ; //
		break ;
		case 7 :
			time_sec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK7_SEC ) ; //
			time_nsec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK7_NS ) ; //
		break ;
		case 0 :
			time_sec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK8_SEC ) ; //
			time_nsec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK8_NS ) ; //
		break ;
		default :
			time_sec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK1_SEC ) ; //
			time_nsec_tmp = IORD( AVB_FRM_WT_BASE[ch_number] ,AVB_FRM_BK1_NS ) ; //
		break ;
	}
	G_Bank_Status[ch_number].sec[wt_bank_status] = time_sec_tmp ;
	G_Bank_Status[ch_number].nsec[wt_bank_status] = time_nsec_tmp ;
	G_Bank_Status[ch_number].wt_bank_no = wt_bank_status ;
	for ( i = 0 ; i < 4 ; i ++ ) {
		if ( G_Bank_Status[i].bank_flag != 0x00 ) {
			G_Bank_Status[i].bank_flag = G_Bank_Status[i].bank_flag - 1;
		}
	}
	G_Bank_Status[ch_number].bank_flag = 16 ;
	
	return wt_bank_status ;
}

//=====================//
// Output bank setting //
//=====================//
void set_output ( alt_u8 base_bank , alt_u8 out_ch) {
	G_Bank_Status[out_ch].out_bank_no = G_Bank_Status[out_ch].wt_bank_no ;

	switch ( G_Bank_Status[out_ch].out_bank_no ) {
		case 0 :
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x04 ,CH_OFFSET_ADR[out_ch] + BANK1_ADR ) ;
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x0B ,CH_OFFSET_ADR[out_ch] + BANK1_ADR ) ;
			IOWR( AVB_FRM_WT_BASE[out_ch] ,AVB_FRM_RD_BANK ,0x00000000 ) ;
//					printf( "CH%d:%d " ,i ,0 ) ;
		break ;
		case 1 :
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x04 ,CH_OFFSET_ADR[out_ch] + BANK2_ADR ) ;
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x0B ,CH_OFFSET_ADR[out_ch] + BANK2_ADR ) ;
			IOWR( AVB_FRM_WT_BASE[out_ch] ,AVB_FRM_RD_BANK ,0x00000001 ) ;
//					printf( "CH%d:%d " ,i ,1 ) ;
		break ;
		case 2 :
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x04 ,CH_OFFSET_ADR[out_ch] + BANK3_ADR ) ;
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x0B ,CH_OFFSET_ADR[out_ch] + BANK3_ADR ) ;
			IOWR( AVB_FRM_WT_BASE[out_ch] ,AVB_FRM_RD_BANK ,0x00000002 ) ;
//					printf( "CH%d:%d " ,i ,2 ) ;
		break ;
		case 3 :
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x04 ,CH_OFFSET_ADR[out_ch] + BANK4_ADR ) ;
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x0B ,CH_OFFSET_ADR[out_ch] + BANK4_ADR ) ;
			IOWR( AVB_FRM_WT_BASE[out_ch] ,AVB_FRM_RD_BANK ,0x00000003 ) ;
//					printf( "CH%d:%d " ,i ,3 ) ;
		break ;
		case 4 :
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x04 ,CH_OFFSET_ADR[out_ch] + BANK5_ADR ) ;
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x0B ,CH_OFFSET_ADR[out_ch] + BANK5_ADR ) ;
			IOWR( AVB_FRM_WT_BASE[out_ch] ,AVB_FRM_RD_BANK ,0x00000004 ) ;
//					printf( "CH%d:%d " ,i ,4 ) ;
		break ;
		case 5 :
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x04 ,CH_OFFSET_ADR[out_ch] + BANK6_ADR ) ;
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x0B ,CH_OFFSET_ADR[out_ch] + BANK6_ADR ) ;
			IOWR( AVB_FRM_WT_BASE[out_ch] ,AVB_FRM_RD_BANK ,0x00000005 ) ;
//					printf( "CH%d:%d " ,i ,5 ) ;
		break ;
		case 6 :
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x04 ,CH_OFFSET_ADR[out_ch] + BANK7_ADR ) ;
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x0B ,CH_OFFSET_ADR[out_ch] + BANK7_ADR ) ;
			IOWR( AVB_FRM_WT_BASE[out_ch] ,AVB_FRM_RD_BANK ,0x00000006 ) ;
//					printf( "CH%d:%d " ,i ,6 ) ;
		break ;
		case 7 :
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x04 ,CH_OFFSET_ADR[out_ch] + BANK8_ADR ) ;
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x0B ,CH_OFFSET_ADR[out_ch] + BANK8_ADR ) ;
			IOWR( AVB_FRM_WT_BASE[out_ch] ,AVB_FRM_RD_BANK ,0x00000007 ) ;
//					printf( "CH%d:%d " ,i ,7 ) ;
		break ;
		default :
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x04 ,CH_OFFSET_ADR[out_ch] + BANK1_ADR ) ;
			IOWR( AVB_FRM_RD_BASE[out_ch] ,0x0B ,CH_OFFSET_ADR[out_ch] + BANK1_ADR ) ;
			IOWR( AVB_FRM_WT_BASE[out_ch] ,AVB_FRM_RD_BANK ,0x00000008 ) ;
//					printf( "CH%d:X " ,i  ) ;
		break ;
	}
	IOWR( AVB_FRM_RD_BASE[out_ch] ,0x00 ,0x00000001 ) ;
	
}

//======================//
// display port control //
//======================//
void display_port_control( ) {
	// Serve Syslib periodic tasks
	btc_dptx_syslib_monitor();

	// Dump AUX channel traffic
	//bitec_dp_dump_aux_debug(AUX_RX_DEBUG_FIFO_IN_CSR_BASE, AUX_RX_DEBUG_FIFO_OUT_BASE, 1);
	bitec_dp_dump_aux_debug(AUX_TX_DEBUG_FIFO_IN_CSR_BASE, AUX_TX_DEBUG_FIFO_OUT_BASE, 0);

}

//======//
// main //
//======//
int main() {

	alt_u8			board_no ;
	alt_u8			pb1 = 0 ;
	alt_u8			pb1_tmp = 0 ;
	alt_u8			pb1_state = 0 ;
	alt_u8			pb2 = 0 ;
	alt_u8			pb2_tmp = 0 ;
	alt_u8			pb2_state = 0 ;
	alt_u8			dp_wait_flag = 0 ;
	unsigned int	tout ;
	alt_u8			dip_state ;

	board_no = ( 0x01 & (alt_u8)(IORD( DIP_SW_BASE , 0x00 )) ) ;

	IOWR( LED_BASE , 0x00,0x0000000F ) ;			//
	netwrok_init( board_no ) ;
	stream_init( board_no ) ;
	display_init( board_no ) ;
	display_port_init( ) ;
	set_output ( G_Base_Bank[0] , 0 ) ;				// o͐ݒ
	printf( "Initial END \n" ) ;
	while(1) {
		pb1_tmp = pb1 ;
		pb1 = ( 0x02 & (alt_u8)(IORD( PUSH_SW_BASE , 0x00 )) ) ;
		pb1_state = ( pb1 < pb1_tmp ) ? 1 : 0 ;
		pb2_tmp = pb2 ;
		pb2 = ( 0x04 & (alt_u8)(IORD( PUSH_SW_BASE , 0x00 )) ) ;
		pb2_state = ( pb2 < pb2_tmp ) ? 1 : 0 ;
		
		if ( (IORD( AVB_FRM_WT_BASE[0] ,0x25 )) == 0x1 ) {	// CHn Frame write interrupt
			G_Base_Bank[0] = rd_bank_status ( 0 ) ;			// 荞݂̂`l̃Xe[^Xǂݍ
			set_output ( G_Base_Bank[0] , 0 ) ;				// o͐ݒ
		}

		if ( dp_wait_flag ) {
			if ( alt_timestamp() < tout ) {
				display_port_control( ) ;
				dp_wait_flag = 0 ;
			}
		}
		else {
			alt_timestamp_start( );
			tout = alt_timestamp_freq( ) / 2 ;
			dp_wait_flag = 1 ;
		}
	}
}
