// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_frm_wt_csr.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Ryosuke Yamamoto
// DATE			: 03/10/2014
// =============================================================================
// DESCRIPTION	: 
//
// =============================================================================
// REVISION HISTORY	:
//	v1.0	: Initial version release
//
// =============================================================================
// PARAMETERS	:
//
// =============================================================================
// I/O PORTS	:
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none

module	avb_frm_wt_csr (
	clk						,
	rst_n					,

	address_i				,
	byteenable_i			,
	read_i					,
	write_i					,
	readdatavalid_o			,
	readdata_o				,
	writedata_i				,
	waitrequest_o			,

	frame_number_i			,
	packet_number_i			,
	total_packet_count_i	,
	time_stamp_sec_i		,
	time_stamp_ns_i			,
	fifo_wt_start_i			,
	fifo_wt_end_i			,
	mem_wt_end_i			,

	srst_o					,
	packet_size_o			,
	src_mac_adr_o			,
	acc_adr_o				,
	acc_adr_valid_o			,

	rx_frame_end_int_o

) ;
	
// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter	AVMS_ADR_BIT	= 6 ;
	parameter	AVMS_DATA_BIT	= 32 ;
	parameter	AVMS_BE_BIT		= AVMS_DATA_BIT / 8 ;
	parameter	AVMM_ADR_BIT	= 27 ;

	parameter	MAC_ADR_BIT		= 48 ;
	parameter	TYPE_BIT		= 16 ;
	parameter	FRM_NUM_BIT		= 16 ;
	parameter	PACKET_NUM_BIT	= 32 ;
	parameter	T_STP_SEC_BIT	= 48 ;
	parameter	T_STP_NS_BIT	= 32 ;

	parameter	PACKET_SIZE_BIT	= 11 ;

	parameter	VLD_00		= 32'h0000_0001 ;	// srst
	parameter	VLD_04		= 32'h0000_FFFF ;	// dest adr(top)
	parameter	VLD_08		= 32'hFFFF_FFFF ;	// dest adr(Low)
	parameter	VLD_0C		= { { (32-PACKET_SIZE_BIT){1'b0} } , { PACKET_SIZE_BIT{1'b1} } } ;	// packet size
	parameter	VLD_10		= 32'hFFFF_FFFF ;	// bank1 offset address
	parameter	VLD_14		= 32'h0000_FFFF ;	// bank1 frame number
	parameter	VLD_18		= 32'hFFFF_FFFF ;	// bank1 time sec (4byte)
	parameter	VLD_1C		= 32'hFFFF_FFFF ;	// bank1 time nsec (4byte)
	parameter	VLD_20		= 32'hFFFF_FFFF ;	// bank2 offset address
	parameter	VLD_24		= 32'h0000_FFFF ;	// bank2 frame number
	parameter	VLD_28		= 32'hFFFF_FFFF ;	// bank2 time sec (4byte)
	parameter	VLD_2C		= 32'hFFFF_FFFF ;	// bank2 time nsec (4byte)
	parameter	VLD_30		= 32'hFFFF_FFFF ;	// bank3 offset address
	parameter	VLD_34		= 32'h0000_FFFF ;	// bank3 frame number
	parameter	VLD_38		= 32'hFFFF_FFFF ;	// bank3 time sec (4byte)
	parameter	VLD_3C		= 32'hFFFF_FFFF ;	// bank3 time nsec (4byte)
	parameter	VLD_40		= 32'hFFFF_FFFF ;	// bank4 offset address
	parameter	VLD_44		= 32'h0000_FFFF ;	// bank4 frame number
	parameter	VLD_48		= 32'hFFFF_FFFF ;	// bank4 time sec (4byte)
	parameter	VLD_4C		= 32'hFFFF_FFFF ;	// bank4 time nsec (4byte)
	parameter	VLD_50		= 32'hFFFF_FFFF ;	// bank5 offset address
	parameter	VLD_54		= 32'h0000_FFFF ;	// bank5 frame number
	parameter	VLD_58		= 32'hFFFF_FFFF ;	// bank5 time sec (4byte)
	parameter	VLD_5C		= 32'hFFFF_FFFF ;	// bank5 time nsec (4byte)
	parameter	VLD_60		= 32'hFFFF_FFFF ;	// bank6 offset address
	parameter	VLD_64		= 32'h0000_FFFF ;	// bank6 frame number
	parameter	VLD_68		= 32'hFFFF_FFFF ;	// bank6 time sec (4byte)
	parameter	VLD_6C		= 32'hFFFF_FFFF ;	// bank6 time nsec (4byte)
	parameter	VLD_70		= 32'hFFFF_FFFF ;	// bank7 offset address
	parameter	VLD_74		= 32'h0000_FFFF ;	// bank7 frame number
	parameter	VLD_78		= 32'hFFFF_FFFF ;	// bank7 time sec (4byte)
	parameter	VLD_7C		= 32'hFFFF_FFFF ;	// bank7 time nsec (4byte)
	parameter	VLD_80		= 32'hFFFF_FFFF ;	// bank8 offset address
	parameter	VLD_84		= 32'h0000_FFFF ;	// bank8 frame number
	parameter	VLD_88		= 32'hFFFF_FFFF ;	// bank8 time sec (4byte)
	parameter	VLD_8C		= 32'hFFFF_FFFF ;	// bank8 time nsec (4byte)
	parameter	VLD_90		= 32'h0000_0001 ;	// int mask
	parameter	VLD_94		= 32'h0000_0001 ;	// int
	parameter	VLD_98		= 32'h0000_0007 ;	// write bank
	parameter	VLD_9C		= 32'h0000_0007 ;	// read bank

	parameter	INI_00		= 32'h0000_0000 ;
	parameter	INI_04		= 32'h0000_0000 ;
	parameter	INI_08		= 32'h0000_0000 ;
	parameter	INI_0C		= 32'h0000_0400 ;
	parameter	INI_10		= 32'h0000_0000 ;
	parameter	INI_14		= 32'h0000_0000 ;
	parameter	INI_18		= 32'h0000_0000 ;
	parameter	INI_1C		= 32'h0000_0000 ;
	parameter	INI_20		= 32'h0000_0000 ;
	parameter	INI_24		= 32'h0000_0000 ;
	parameter	INI_28		= 32'h0000_0000 ;
	parameter	INI_2C		= 32'h0000_0000 ;
	parameter	INI_30		= 32'h0000_0000 ;
	parameter	INI_34		= 32'h0000_0000 ;
	parameter	INI_38		= 32'h0000_0000 ;
	parameter	INI_3C		= 32'h0000_0000 ;
	parameter	INI_40		= 32'h0000_0000 ;
	parameter	INI_44		= 32'h0000_0000 ;
	parameter	INI_48		= 32'h0000_0000 ;
	parameter	INI_4C		= 32'h0000_0000 ;
	parameter	INI_50		= 32'h0000_0000 ;
	parameter	INI_54		= 32'h0000_0000 ;
	parameter	INI_58		= 32'h0000_0000 ;
	parameter	INI_5C		= 32'h0000_0000 ;
	parameter	INI_60		= 32'h0000_0000 ;
	parameter	INI_64		= 32'h0000_0000 ;
	parameter	INI_68		= 32'h0000_0000 ;
	parameter	INI_6C		= 32'h0000_0000 ;
	parameter	INI_70		= 32'h0000_0000 ;
	parameter	INI_74		= 32'h0000_0000 ;
	parameter	INI_78		= 32'h0000_0000 ;
	parameter	INI_7C		= 32'h0000_0000 ;
	parameter	INI_80		= 32'h0000_0000 ;
	parameter	INI_84		= 32'h0000_0000 ;
	parameter	INI_88		= 32'h0000_0000 ;
	parameter	INI_8C		= 32'h0000_0000 ;
	parameter	INI_90		= 32'h0000_0001 ;
	parameter	INI_94		= 32'h0000_0000 ;
	parameter	INI_98		= 32'h0000_0000 ;
	parameter	INI_9C		= 32'h0000_0000 ;

	parameter	PLS_00		= 32'h0000_0000 ;
	parameter	PLS_04		= 32'h0000_0000 ;
	parameter	PLS_08		= 32'h0000_0000 ;
	parameter	PLS_0C		= 32'h0000_0000 ;
	parameter	PLS_10		= 32'h0000_0000 ;
	parameter	PLS_14		= 32'h0000_0000 ;
	parameter	PLS_18		= 32'h0000_0000 ;
	parameter	PLS_1C		= 32'h0000_0000 ;
	parameter	PLS_20		= 32'h0000_0000 ;
	parameter	PLS_24		= 32'h0000_0000 ;
	parameter	PLS_28		= 32'h0000_0000 ;
	parameter	PLS_2C		= 32'h0000_0000 ;
	parameter	PLS_30		= 32'h0000_0000 ;
	parameter	PLS_34		= 32'h0000_0000 ;
	parameter	PLS_38		= 32'h0000_0000 ;
	parameter	PLS_3C		= 32'h0000_0000 ;
	parameter	PLS_40		= 32'h0000_0000 ;
	parameter	PLS_44		= 32'h0000_0000 ;
	parameter	PLS_48		= 32'h0000_0000 ;
	parameter	PLS_4C		= 32'h0000_0000 ;
	parameter	PLS_50		= 32'h0000_0000 ;
	parameter	PLS_54		= 32'h0000_0000 ;
	parameter	PLS_58		= 32'h0000_0000 ;
	parameter	PLS_5C		= 32'h0000_0000 ;
	parameter	PLS_60		= 32'h0000_0000 ;
	parameter	PLS_64		= 32'h0000_0000 ;
	parameter	PLS_68		= 32'h0000_0000 ;
	parameter	PLS_6C		= 32'h0000_0000 ;
	parameter	PLS_70		= 32'h0000_0000 ;
	parameter	PLS_74		= 32'h0000_0000 ;
	parameter	PLS_78		= 32'h0000_0000 ;
	parameter	PLS_7C		= 32'h0000_0000 ;
	parameter	PLS_80		= 32'h0000_0000 ;
	parameter	PLS_84		= 32'h0000_0000 ;
	parameter	PLS_88		= 32'h0000_0000 ;
	parameter	PLS_8C		= 32'h0000_0000 ;
	parameter	PLS_90		= 32'h0000_0000 ;
	parameter	PLS_94		= 32'h0000_0000 ;
	parameter	PLS_98		= 32'h0000_0000 ;
	parameter	PLS_9C		= 32'h0000_0000 ;

	parameter	READ_WRITE	= 0 ;
	parameter	WRITE_CLEAR	= 1 ;
	parameter	READ_ONLY	= 2 ;
	parameter	READ_CLEAR	= 3 ;

	parameter	TYP_00		= READ_WRITE ;
	parameter	TYP_04		= READ_WRITE ;
	parameter	TYP_08		= READ_WRITE ;
	parameter	TYP_0C		= READ_WRITE ;
	parameter	TYP_10		= READ_WRITE ;
	parameter	TYP_14		= READ_ONLY ;
	parameter	TYP_18		= READ_ONLY ;
	parameter	TYP_1C		= READ_ONLY ;
	parameter	TYP_20		= READ_WRITE ;
	parameter	TYP_24		= READ_ONLY ;
	parameter	TYP_28		= READ_ONLY ;
	parameter	TYP_2C		= READ_ONLY ;
	parameter	TYP_30		= READ_WRITE ;
	parameter	TYP_34		= READ_ONLY ;
	parameter	TYP_38		= READ_ONLY ;
	parameter	TYP_3C		= READ_ONLY ;
	parameter	TYP_40		= READ_WRITE ;
	parameter	TYP_44		= READ_ONLY ;
	parameter	TYP_48		= READ_ONLY ;
	parameter	TYP_4C		= READ_ONLY ;
	parameter	TYP_50		= READ_WRITE ;
	parameter	TYP_54		= READ_ONLY ;
	parameter	TYP_58		= READ_ONLY ;
	parameter	TYP_5C		= READ_ONLY ;
	parameter	TYP_60		= READ_WRITE ;
	parameter	TYP_64		= READ_ONLY ;
	parameter	TYP_68		= READ_ONLY ;
	parameter	TYP_6C		= READ_ONLY ;
	parameter	TYP_70		= READ_WRITE ;
	parameter	TYP_74		= READ_ONLY ;
	parameter	TYP_78		= READ_ONLY ;
	parameter	TYP_7C		= READ_ONLY ;
	parameter	TYP_80		= READ_WRITE ;
	parameter	TYP_84		= READ_ONLY ;
	parameter	TYP_88		= READ_ONLY ;
	parameter	TYP_8C		= READ_ONLY ;
	parameter	TYP_90		= READ_WRITE ;
	parameter	TYP_94		= WRITE_CLEAR ;
	parameter	TYP_98		= READ_ONLY ;
	parameter	TYP_9C		= READ_WRITE ;

	parameter	WPO_00		= 0 ;
	parameter	WPO_04		= 0 ;
	parameter	WPO_08		= 0 ;
	parameter	WPO_0C		= 0 ;
	parameter	WPO_10		= 0 ;
	parameter	WPO_14		= 0 ;
	parameter	WPO_18		= 0 ;
	parameter	WPO_1C		= 0 ;
	parameter	WPO_20		= 0 ;
	parameter	WPO_24		= 0 ;
	parameter	WPO_28		= 0 ;
	parameter	WPO_2C		= 0 ;
	parameter	WPO_30		= 0 ;
	parameter	WPO_34		= 0 ;
	parameter	WPO_38		= 0 ;
	parameter	WPO_3C		= 0 ;
	parameter	WPO_40		= 0 ;
	parameter	WPO_44		= 0 ;
	parameter	WPO_48		= 0 ;
	parameter	WPO_4C		= 0 ;
	parameter	WPO_50		= 0 ;
	parameter	WPO_54		= 0 ;
	parameter	WPO_58		= 0 ;
	parameter	WPO_5C		= 0 ;
	parameter	WPO_60		= 0 ;
	parameter	WPO_64		= 0 ;
	parameter	WPO_68		= 0 ;
	parameter	WPO_6C		= 0 ;
	parameter	WPO_70		= 0 ;
	parameter	WPO_74		= 0 ;
	parameter	WPO_78		= 0 ;
	parameter	WPO_7C		= 0 ;
	parameter	WPO_80		= 0 ;
	parameter	WPO_84		= 0 ;
	parameter	WPO_88		= 0 ;
	parameter	WPO_8C		= 0 ;
	parameter	WPO_90		= 1 ;
	parameter	WPO_94		= 0 ;
	parameter	WPO_98		= 1 ;
	parameter	WPO_9C		= 0 ;

	parameter	RPO_00		= 1 ;
	parameter	RPO_04		= 1 ;
	parameter	RPO_08		= 1 ;
	parameter	RPO_0C		= 1 ;
	parameter	RPO_10		= 1 ;
	parameter	RPO_14		= 1 ;
	parameter	RPO_18		= 1 ;
	parameter	RPO_1C		= 1 ;
	parameter	RPO_20		= 1 ;
	parameter	RPO_24		= 1 ;
	parameter	RPO_28		= 1 ;
	parameter	RPO_2C		= 1 ;
	parameter	RPO_30		= 1 ;
	parameter	RPO_34		= 1 ;
	parameter	RPO_38		= 1 ;
	parameter	RPO_3C		= 1 ;
	parameter	RPO_40		= 1 ;
	parameter	RPO_44		= 1 ;
	parameter	RPO_48		= 1 ;
	parameter	RPO_4C		= 1 ;
	parameter	RPO_50		= 1 ;
	parameter	RPO_54		= 1 ;
	parameter	RPO_58		= 1 ;
	parameter	RPO_5C		= 1 ;
	parameter	RPO_60		= 1 ;
	parameter	RPO_64		= 1 ;
	parameter	RPO_68		= 1 ;
	parameter	RPO_6C		= 1 ;
	parameter	RPO_70		= 1 ;
	parameter	RPO_74		= 1 ;
	parameter	RPO_78		= 1 ;
	parameter	RPO_7C		= 1 ;
	parameter	RPO_80		= 1 ;
	parameter	RPO_84		= 1 ;
	parameter	RPO_88		= 1 ;
	parameter	RPO_8C		= 1 ;
	parameter	RPO_90		= 1 ;
	parameter	RPO_94		= 1 ;
	parameter	RPO_98		= 1 ;
	parameter	RPO_9C		= 1 ;


// =============================================================================
// PORT DECLARATION
// =============================================================================

	input	wire								clk						;
	input	wire								rst_n					;

	input	wire	[   AVMS_ADR_BIT-1 : 0 ]	address_i				;
	input	wire	[    AVMS_BE_BIT-1 : 0 ]	byteenable_i			;
	input	wire								read_i					;
	input	wire								write_i					;
	output	wire								readdatavalid_o			;
	output	wire	[   AVMS_DATA_BIT-1 : 0 ]	readdata_o				;
	input	wire	[   AVMS_DATA_BIT-1 : 0 ]	writedata_i				;
	output	wire								waitrequest_o			;

	input	wire	[     FRM_NUM_BIT-1 : 0 ]	frame_number_i			;
	input	wire	[  PACKET_NUM_BIT-1 : 0 ]	packet_number_i			;
	input	wire	[  PACKET_NUM_BIT-1 : 0 ]	total_packet_count_i	;
	input	wire	[   T_STP_SEC_BIT-1 : 0 ]	time_stamp_sec_i		;
	input	wire	[    T_STP_NS_BIT-1 : 0 ]	time_stamp_ns_i			;
	input	wire								fifo_wt_start_i			;
	input	wire								fifo_wt_end_i			;
	input	wire								mem_wt_end_i			;

	output	wire								srst_o					;
	output	wire	[ PACKET_SIZE_BIT-1 : 0 ]	packet_size_o			;
	output	wire	[     MAC_ADR_BIT-1 : 0 ]	src_mac_adr_o			;
	output	wire	[  AVMM_ADR_BIT+4-1 : 0 ]	acc_adr_o				;
	output	wire								acc_adr_valid_o			;

	output	wire								rx_frame_end_int_o		;


// =============================================================================
// REG / WIRE DECLARATION
// =============================================================================

	genvar							i					;

	reg		[              31 : 0 ]	state_in			[ 0 : 39 ] ;
	wire	[              79 : 0 ]	reg_in				[ 0 : 39 ] ;
	wire	[              65 : 0 ]	reg_out				[ 0 : 39 ] ;
	reg		[ AVMS_DATA_BIT-1 : 0 ]	rd_data_ff			;
	reg								waitrequest_ff		;
	reg								readdatavalid_ff	;
	
	wire	[   FRM_NUM_BIT-1 : 0 ]	bk1_frm_num			;
	wire	[ T_STP_SEC_BIT-1 : 0 ]	bk1_time_sec		;
	wire	[  T_STP_NS_BIT-1 : 0 ]	bk1_time_ns			;
	wire	[   FRM_NUM_BIT-1 : 0 ]	bk2_frm_num			;
	wire	[ T_STP_SEC_BIT-1 : 0 ]	bk2_time_sec		;
	wire	[  T_STP_NS_BIT-1 : 0 ]	bk2_time_ns			;
	wire	[   FRM_NUM_BIT-1 : 0 ]	bk3_frm_num			;
	wire	[ T_STP_SEC_BIT-1 : 0 ]	bk3_time_sec		;
	wire	[  T_STP_NS_BIT-1 : 0 ]	bk3_time_ns			;
	wire	[   FRM_NUM_BIT-1 : 0 ]	bk4_frm_num			;
	wire	[ T_STP_SEC_BIT-1 : 0 ]	bk4_time_sec		;
	wire	[  T_STP_NS_BIT-1 : 0 ]	bk4_time_ns			;
	wire	[   FRM_NUM_BIT-1 : 0 ]	bk5_frm_num			;
	wire	[ T_STP_SEC_BIT-1 : 0 ]	bk5_time_sec		;
	wire	[  T_STP_NS_BIT-1 : 0 ]	bk5_time_ns			;
	wire	[   FRM_NUM_BIT-1 : 0 ]	bk6_frm_num			;
	wire	[ T_STP_SEC_BIT-1 : 0 ]	bk6_time_sec		;
	wire	[  T_STP_NS_BIT-1 : 0 ]	bk6_time_ns			;
	wire	[   FRM_NUM_BIT-1 : 0 ]	bk7_frm_num			;
	wire	[ T_STP_SEC_BIT-1 : 0 ]	bk7_time_sec		;
	wire	[  T_STP_NS_BIT-1 : 0 ]	bk7_time_ns			;
	wire	[   FRM_NUM_BIT-1 : 0 ]	bk8_frm_num			;
	wire	[ T_STP_SEC_BIT-1 : 0 ]	bk8_time_sec		;
	wire	[  T_STP_NS_BIT-1 : 0 ]	bk8_time_ns			;

	wire							rx_frame_end		;
	wire	[               2 : 0 ]	rx_frame_bank		;

// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================

	//-----------------------//
	// Avalon-MM I/F control //
	//-----------------------//
	generate
		for ( i = 0 ; i < 40 ; i = i + 1 ) begin	: state_in_generate
			always @ ( * ) begin
				case ( i[5:0] )
					6'h05	: state_in[i]	<= { 16'h0000 , bk1_frm_num } ;
					6'h06	: state_in[i]	<= bk1_time_sec[31:0] ;
					6'h07	: state_in[i]	<= bk1_time_ns[31:0] ;
					6'h09	: state_in[i]	<= { 16'h0000 , bk2_frm_num } ;
					6'h0A	: state_in[i]	<= bk2_time_sec[31:0] ;
					6'h0B	: state_in[i]	<= bk2_time_ns[31:0] ;
					6'h0D	: state_in[i]	<= { 16'h0000 , bk3_frm_num } ;
					6'h0E	: state_in[i]	<= bk3_time_sec[31:0] ;
					6'h0F	: state_in[i]	<= bk3_time_ns[31:0] ;
					6'h11	: state_in[i]	<= { 16'h0000 , bk4_frm_num } ;
					6'h12	: state_in[i]	<= bk4_time_sec[31:0] ;
					6'h13	: state_in[i]	<= bk4_time_ns[31:0] ;
					6'h15	: state_in[i]	<= { 16'h0000 , bk5_frm_num } ;
					6'h16	: state_in[i]	<= bk5_time_sec[31:0] ;
					6'h17	: state_in[i]	<= bk5_time_ns[31:0] ;
					6'h19	: state_in[i]	<= { 16'h0000 , bk6_frm_num } ;
					6'h1A	: state_in[i]	<= bk6_time_sec[31:0] ;
					6'h1B	: state_in[i]	<= bk6_time_ns[31:0] ;
					6'h1D	: state_in[i]	<= { 16'h0000 , bk7_frm_num } ;
					6'h1E	: state_in[i]	<= bk7_time_sec[31:0] ;
					6'h1F	: state_in[i]	<= bk7_time_ns[31:0] ;
					6'h21	: state_in[i]	<= { 16'h0000 , bk8_frm_num } ;
					6'h22	: state_in[i]	<= bk8_time_sec[31:0] ;
					6'h23	: state_in[i]	<= bk8_time_ns[31:0] ;
					6'h25	: state_in[i]	<= { 31'h0000_0000 , rx_frame_end } ;
					6'h26	: state_in[i]	<= { 29'h0000_0000 , rx_frame_bank } ;
					default	: state_in[i]	<= 32'h0000_0000 ;
				endcase
			end
			assign reg_in[i]	= { clk ,			// 
								    rst_n ,			// 
								    1'b0 ,			// 
								    1'b1 ,			// 
								    address_i ,		// 
								    write_i ,		// 
								    read_i ,		// 
								    byteenable_i ,	// 
								    state_in[i] ,	// 
								    writedata_i } ;	// 
		end
	endgenerate

	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h00),.INI(INI_00),.VLD(VLD_00),.PLS(PLS_00),.TYP(TYP_00),.WPO(WPO_00),.RPO(RPO_00)) u00H (.reg_i(reg_in[ 0]),.reg_o(reg_out[ 0])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h01),.INI(INI_04),.VLD(VLD_04),.PLS(PLS_04),.TYP(TYP_04),.WPO(WPO_04),.RPO(RPO_04)) u04H (.reg_i(reg_in[ 1]),.reg_o(reg_out[ 1])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h02),.INI(INI_08),.VLD(VLD_08),.PLS(PLS_08),.TYP(TYP_08),.WPO(WPO_08),.RPO(RPO_08)) u08H (.reg_i(reg_in[ 2]),.reg_o(reg_out[ 2])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h03),.INI(INI_0C),.VLD(VLD_0C),.PLS(PLS_0C),.TYP(TYP_0C),.WPO(WPO_0C),.RPO(RPO_0C)) u0CH (.reg_i(reg_in[ 3]),.reg_o(reg_out[ 3])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h04),.INI(INI_10),.VLD(VLD_10),.PLS(PLS_10),.TYP(TYP_10),.WPO(WPO_10),.RPO(RPO_10)) u10H (.reg_i(reg_in[ 4]),.reg_o(reg_out[ 4])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h05),.INI(INI_14),.VLD(VLD_14),.PLS(PLS_14),.TYP(TYP_14),.WPO(WPO_14),.RPO(RPO_14)) u14H (.reg_i(reg_in[ 5]),.reg_o(reg_out[ 5])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h06),.INI(INI_18),.VLD(VLD_18),.PLS(PLS_18),.TYP(TYP_18),.WPO(WPO_18),.RPO(RPO_18)) u18H (.reg_i(reg_in[ 6]),.reg_o(reg_out[ 6])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h07),.INI(INI_1C),.VLD(VLD_1C),.PLS(PLS_1C),.TYP(TYP_1C),.WPO(WPO_1C),.RPO(RPO_1C)) u1CH (.reg_i(reg_in[ 7]),.reg_o(reg_out[ 7])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h08),.INI(INI_20),.VLD(VLD_20),.PLS(PLS_20),.TYP(TYP_20),.WPO(WPO_20),.RPO(RPO_20)) u20H (.reg_i(reg_in[ 8]),.reg_o(reg_out[ 8])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h09),.INI(INI_24),.VLD(VLD_24),.PLS(PLS_24),.TYP(TYP_24),.WPO(WPO_24),.RPO(RPO_24)) u24H (.reg_i(reg_in[ 9]),.reg_o(reg_out[ 9])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h0A),.INI(INI_28),.VLD(VLD_28),.PLS(PLS_28),.TYP(TYP_28),.WPO(WPO_28),.RPO(RPO_28)) u28H (.reg_i(reg_in[10]),.reg_o(reg_out[10])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h0B),.INI(INI_2C),.VLD(VLD_2C),.PLS(PLS_2C),.TYP(TYP_2C),.WPO(WPO_2C),.RPO(RPO_2C)) u2CH (.reg_i(reg_in[11]),.reg_o(reg_out[11])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h0C),.INI(INI_30),.VLD(VLD_30),.PLS(PLS_30),.TYP(TYP_30),.WPO(WPO_30),.RPO(RPO_30)) u30H (.reg_i(reg_in[12]),.reg_o(reg_out[12])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h0D),.INI(INI_34),.VLD(VLD_34),.PLS(PLS_34),.TYP(TYP_34),.WPO(WPO_34),.RPO(RPO_34)) u34H (.reg_i(reg_in[13]),.reg_o(reg_out[13])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h0E),.INI(INI_38),.VLD(VLD_38),.PLS(PLS_38),.TYP(TYP_38),.WPO(WPO_38),.RPO(RPO_38)) u38H (.reg_i(reg_in[14]),.reg_o(reg_out[14])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h0F),.INI(INI_3C),.VLD(VLD_3C),.PLS(PLS_3C),.TYP(TYP_3C),.WPO(WPO_3C),.RPO(RPO_3C)) u3CH (.reg_i(reg_in[15]),.reg_o(reg_out[15])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h10),.INI(INI_40),.VLD(VLD_40),.PLS(PLS_40),.TYP(TYP_40),.WPO(WPO_40),.RPO(RPO_40)) u40H (.reg_i(reg_in[16]),.reg_o(reg_out[16])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h11),.INI(INI_44),.VLD(VLD_44),.PLS(PLS_44),.TYP(TYP_44),.WPO(WPO_44),.RPO(RPO_44)) u44H (.reg_i(reg_in[17]),.reg_o(reg_out[17])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h12),.INI(INI_48),.VLD(VLD_48),.PLS(PLS_48),.TYP(TYP_48),.WPO(WPO_48),.RPO(RPO_48)) u48H (.reg_i(reg_in[18]),.reg_o(reg_out[18])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h13),.INI(INI_4C),.VLD(VLD_4C),.PLS(PLS_4C),.TYP(TYP_4C),.WPO(WPO_4C),.RPO(RPO_4C)) u4CH (.reg_i(reg_in[19]),.reg_o(reg_out[19])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h14),.INI(INI_50),.VLD(VLD_50),.PLS(PLS_50),.TYP(TYP_50),.WPO(WPO_50),.RPO(RPO_50)) u50H (.reg_i(reg_in[20]),.reg_o(reg_out[20])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h15),.INI(INI_54),.VLD(VLD_54),.PLS(PLS_54),.TYP(TYP_54),.WPO(WPO_54),.RPO(RPO_54)) u54H (.reg_i(reg_in[21]),.reg_o(reg_out[21])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h16),.INI(INI_58),.VLD(VLD_58),.PLS(PLS_58),.TYP(TYP_58),.WPO(WPO_58),.RPO(RPO_58)) u58H (.reg_i(reg_in[22]),.reg_o(reg_out[22])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h17),.INI(INI_5C),.VLD(VLD_5C),.PLS(PLS_5C),.TYP(TYP_5C),.WPO(WPO_5C),.RPO(RPO_5C)) u5CH (.reg_i(reg_in[23]),.reg_o(reg_out[23])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h18),.INI(INI_60),.VLD(VLD_60),.PLS(PLS_60),.TYP(TYP_60),.WPO(WPO_60),.RPO(RPO_60)) u60H (.reg_i(reg_in[24]),.reg_o(reg_out[24])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h19),.INI(INI_64),.VLD(VLD_64),.PLS(PLS_64),.TYP(TYP_64),.WPO(WPO_64),.RPO(RPO_64)) u64H (.reg_i(reg_in[25]),.reg_o(reg_out[25])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h1A),.INI(INI_68),.VLD(VLD_68),.PLS(PLS_68),.TYP(TYP_68),.WPO(WPO_68),.RPO(RPO_68)) u68H (.reg_i(reg_in[26]),.reg_o(reg_out[26])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h1B),.INI(INI_6C),.VLD(VLD_6C),.PLS(PLS_6C),.TYP(TYP_6C),.WPO(WPO_6C),.RPO(RPO_6C)) u6CH (.reg_i(reg_in[27]),.reg_o(reg_out[27])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h1C),.INI(INI_70),.VLD(VLD_70),.PLS(PLS_70),.TYP(TYP_70),.WPO(WPO_70),.RPO(RPO_70)) u70H (.reg_i(reg_in[28]),.reg_o(reg_out[28])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h1D),.INI(INI_74),.VLD(VLD_74),.PLS(PLS_74),.TYP(TYP_74),.WPO(WPO_74),.RPO(RPO_74)) u74H (.reg_i(reg_in[29]),.reg_o(reg_out[29])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h1E),.INI(INI_78),.VLD(VLD_78),.PLS(PLS_78),.TYP(TYP_78),.WPO(WPO_78),.RPO(RPO_78)) u78H (.reg_i(reg_in[30]),.reg_o(reg_out[30])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h1F),.INI(INI_7C),.VLD(VLD_7C),.PLS(PLS_7C),.TYP(TYP_7C),.WPO(WPO_7C),.RPO(RPO_7C)) u7CH (.reg_i(reg_in[31]),.reg_o(reg_out[31])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h20),.INI(INI_80),.VLD(VLD_80),.PLS(PLS_80),.TYP(TYP_80),.WPO(WPO_80),.RPO(RPO_80)) u80H (.reg_i(reg_in[32]),.reg_o(reg_out[32])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h21),.INI(INI_84),.VLD(VLD_84),.PLS(PLS_84),.TYP(TYP_84),.WPO(WPO_84),.RPO(RPO_84)) u84H (.reg_i(reg_in[33]),.reg_o(reg_out[33])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h22),.INI(INI_88),.VLD(VLD_88),.PLS(PLS_88),.TYP(TYP_88),.WPO(WPO_88),.RPO(RPO_88)) u88H (.reg_i(reg_in[34]),.reg_o(reg_out[34])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h23),.INI(INI_8C),.VLD(VLD_8C),.PLS(PLS_8C),.TYP(TYP_8C),.WPO(WPO_8C),.RPO(RPO_8C)) u8CH (.reg_i(reg_in[35]),.reg_o(reg_out[35])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h24),.INI(INI_90),.VLD(VLD_90),.PLS(PLS_90),.TYP(TYP_90),.WPO(WPO_90),.RPO(RPO_90)) u90H (.reg_i(reg_in[36]),.reg_o(reg_out[36])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h25),.INI(INI_94),.VLD(VLD_94),.PLS(PLS_94),.TYP(TYP_94),.WPO(WPO_94),.RPO(RPO_94)) u94H (.reg_i(reg_in[37]),.reg_o(reg_out[37])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h26),.INI(INI_98),.VLD(VLD_98),.PLS(PLS_98),.TYP(TYP_98),.WPO(WPO_98),.RPO(RPO_98)) u98H (.reg_i(reg_in[38]),.reg_o(reg_out[38])) ;
	avb_frm_wt_reg_ff #(.ADW(6),.DTW(32),.ADR(6'h27),.INI(INI_9C),.VLD(VLD_9C),.PLS(PLS_9C),.TYP(TYP_9C),.WPO(WPO_9C),.RPO(RPO_9C)) u9CH (.reg_i(reg_in[39]),.reg_o(reg_out[39])) ;

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			rd_data_ff	<= { AVMS_DATA_BIT{1'b0} } ;
		end
		else begin
			rd_data_ff	<= reg_out[address_i][AVMS_DATA_BIT-1:0] ;
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			waitrequest_ff	<= 1'b1 ;
		end
		else begin
			if ( !waitrequest_ff ) begin
				waitrequest_ff	<= 1'b1 ;
			end
			else if ( read_i || write_i ) begin
				waitrequest_ff	<= 1'b0 ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			readdatavalid_ff	<= 1'b0 ;
		end
		else begin
			readdatavalid_ff	<= ~waitrequest_ff & read_i ;
		end
	end

	//--------------//
	// bank control //
	//--------------//
	avb_frm_wt_bank_ctl
		# (
			.AVMM_ADR_BIT			( AVMM_ADR_BIT			) 
		)
		u_avb_frm_wt_bank_ctl (
			.clk					( clk					) ,
			.rst_n					( rst_n					) ,
			.srst					( reg_out[0][0]			) ,

			.packet_size_i			( reg_out[3][10:0]		) ,
			.frame_number_i			( frame_number_i		) ,
			.packet_number_i		( packet_number_i		) ,
			.total_packet_count_i	( total_packet_count_i	) ,
			.time_stamp_sec_i		( time_stamp_sec_i		) ,
			.time_stamp_ns_i		( time_stamp_ns_i		) ,
			.fifo_wt_start_i		( fifo_wt_start_i		) ,
			.fifo_wt_end_i			( fifo_wt_end_i			) ,
			.mem_wt_end_i			( mem_wt_end_i			) ,

			.bk1_adr_i				( reg_out[4][31:0]		) ,
			.bk2_adr_i				( reg_out[8][31:0]		) ,
			.bk3_adr_i				( reg_out[12][31:0]		) ,
			.bk4_adr_i				( reg_out[16][31:0]		) ,
			.bk5_adr_i				( reg_out[20][31:0]		) ,
			.bk6_adr_i				( reg_out[24][31:0]		) ,
			.bk7_adr_i				( reg_out[28][31:0]		) ,
			.bk8_adr_i				( reg_out[32][31:0]		) ,
			.read_bank_i			( reg_out[39][2:0]		) ,

			.bk1_frm_num_o			( bk1_frm_num			) ,
			.bk1_time_sec_o			( bk1_time_sec			) ,
			.bk1_time_ns_o			( bk1_time_ns			) ,
			.bk2_frm_num_o			( bk2_frm_num			) ,
			.bk2_time_sec_o			( bk2_time_sec			) ,
			.bk2_time_ns_o			( bk2_time_ns			) ,
			.bk3_frm_num_o			( bk3_frm_num			) ,
			.bk3_time_sec_o			( bk3_time_sec			) ,
			.bk3_time_ns_o			( bk3_time_ns			) ,
			.bk4_frm_num_o			( bk4_frm_num			) ,
			.bk4_time_sec_o			( bk4_time_sec			) ,
			.bk4_time_ns_o			( bk4_time_ns			) ,
			.bk5_frm_num_o			( bk5_frm_num			) ,
			.bk5_time_sec_o			( bk5_time_sec			) ,
			.bk5_time_ns_o			( bk5_time_ns			) ,
			.bk6_frm_num_o			( bk6_frm_num			) ,
			.bk6_time_sec_o			( bk6_time_sec			) ,
			.bk6_time_ns_o			( bk6_time_ns			) ,
			.bk7_frm_num_o			( bk7_frm_num			) ,
			.bk7_time_sec_o			( bk7_time_sec			) ,
			.bk7_time_ns_o			( bk7_time_ns			) ,
			.bk8_frm_num_o			( bk8_frm_num			) ,
			.bk8_time_sec_o			( bk8_time_sec			) ,
			.bk8_time_ns_o			( bk8_time_ns			) ,

			.rx_frame_end_o			( rx_frame_end			) ,
			.rx_frame_bank_o		( rx_frame_bank			) ,
			.acc_adr_o				( acc_adr_o				) ,
			.acc_adr_valid_o		( acc_adr_valid_o		)
		) ;


	//---------------//
	// output singal //
	//---------------//
	// Avalon-MM
	assign	readdatavalid_o		= readdatavalid_ff ;
	assign	readdata_o			= rd_data_ff ;
	assign	waitrequest_o		= waitrequest_ff ;

	// others
	assign	srst_o				= reg_out[0][0] ;
	assign	src_mac_adr_o		= { reg_out[1][15:0] , reg_out[2][AVMS_DATA_BIT-1:0] } ;
	assign	packet_size_o		= reg_out[3][PACKET_SIZE_BIT-1:0] ;
	assign	rx_frame_end_int_o	= ~reg_out[36][0] & reg_out[37][0] ;


endmodule

`default_nettype wire
